﻿/**
 * 文字列計算クラス CalcInfix
 * [author] kujirahand (http://kujirahand.com)
 * [date]   2009-01-11
 * @example 文字列で書かれた式を計算する
 * var n:Number = CalcInfix.eval("1+floor(2^3/5)");
 * trace(n);
 */
package {
	
	
    public class CalcInfix
    {
		static private const VARIABLE:String = "abcdefghijklmnopqrstuvwxyz";

		public var targetString:String;
        private var items:Array;
		public var value:Number;
        private var polish:Array = [];
		
		private var initRand:Number;
		
        private function expression():void {
            //--- 中置記法を逆ポーランドに変換
            var stack:Array = [];
            
			var priority:Object = {
                'func':4, 'num':4,
                '^':3, '*':2, '/':2, '%':2, '+':1, '-':1,
                '(':0
            };

            var getPriority:Function = function(s:String):int {
                if (priority[s] != undefined) {
                    return priority[s];
                }
                if (s.match("^\w+$") != null) {
                    return priority["func"];
                }
                return priority["num"];
            };
            
			var top:String;
            
			for each (var tmp:String in items) {
                if (tmp == "" || tmp == " ") continue;
            
				if (tmp == "(") {
                    stack.push(tmp);
                }
                else if (tmp == ")") {
                    while (stack.length > 0) {
                        top = stack[stack.length - 1];
                        if (top != "(") {
                            polish.push(stack.pop());
                        } else break;
                    }
                    stack.pop();// skip '('

                }
                else {
                    while (stack.length > 0) {
                        top = stack[stack.length - 1];
                        if (getPriority(tmp) <= getPriority(top)) {
                            polish.push(stack.pop());
                        } else break;
                    }
                    stack.push(tmp);
                }
                // DEBUG
                //trace("polish=", polish.join(" "));
                //trace("stack =", stack.join(" "));
                //trace("---");
            }
            
			while (stack.length > 0) { polish.push(stack.pop()); }
            
        }
		
        public function eval():Number
        {
            var stack:Array = [];
            var v1:Number, v2:Number;
            var getStack2:Function = function():void {
                v2 = Number(stack.pop());
                v1 = Number(stack.pop());
            };
            var getStack1:Function = function(): void {
                v1 = Number(stack.pop());
            };
            for each(var i:String in polish) {
                switch(i) {
                    // 演算子
                    case '+': getStack2(); stack.push(v1 + v2); break;
                    case '-': getStack2(); stack.push(v1 - v2); break;
                    case '*': getStack2(); stack.push(v1 * v2); break;
                    case '/': getStack2(); stack.push(v1 / v2); break;
                    case '%': getStack2(); stack.push(v1 % v2); break;
                    case '^': getStack2(); stack.push(Math.pow(v1, v2)); break;

                    // 関数
                    case 'FLOOR': getStack1(); stack.push(Math.floor(v1)); break;
                    case 'CEIL':  getStack1(); stack.push(Math.ceil(v1)); break;
                    case 'RAND':  getStack1(); stack.push(Math.random()*v1); break;
                    case 'SIN': getStack1(); stack.push(Math.sin(v1 * Math.PI / 180)); break;
                    case 'COS': getStack1(); stack.push(Math.cos(v1 * Math.PI / 180)); break;
                    case 'TAN': getStack1(); stack.push(Math.tan(v1 * Math.PI / 180)); break;
                    case 'ASIN': getStack1(); stack.push(Math.asin(v1) * 180 / Math.PI); break;
                    case 'ACOS': getStack1(); stack.push(Math.acos(v1) * 180 / Math.PI); break;
                    case 'ATAN': getStack1(); stack.push(Math.atan(v1) * 180 / Math.PI); break;
                    case 'ABS': getStack1(); stack.push(Math.abs(v1)); break;
                    case 'EXP': getStack1(); stack.push(Math.exp(v1)); break;
                    case 'LN': getStack1(); stack.push(Math.log(v1)); break;
                    case 'SQRT': getStack1(); stack.push(Math.sqrt(v1)); break;
                    case 'ROUND': getStack1(); stack.push(Math.round(v1)); break;

                    case 'IRAND':
						getStack1();
						if (Stimulus.t == 0)
							stack.push(initRand = Math.random() * v1);
						else
							stack.push(initRand);							
						break;

//                    case 'rect': getStack1(); stack.push(Math.sin(v1 * Math.PI / 180) > 0 ? 1 : 0); break;

					//時間変数
                    case 't': getStack1(); stack.push(Stimulus.t); break;

					//グローバル変数
                    case 'a': getStack1(); stack.push(GlobalVariables.value('a')); break;
                    case 'b': getStack1(); stack.push(GlobalVariables.value('b')); break;
                    case 'c': getStack1(); stack.push(GlobalVariables.value('c')); break;
                    case 'd': getStack1(); stack.push(GlobalVariables.value('d')); break;
                    case 'f': getStack1(); stack.push(GlobalVariables.value('f')); break;
                    case 'g': getStack1(); stack.push(GlobalVariables.value('g')); break;
                    case 'i': getStack1(); stack.push(GlobalVariables.value('i')); break;
                    case 'j': getStack1(); stack.push(GlobalVariables.value('j')); break;
                    case 'k': getStack1(); stack.push(GlobalVariables.value('k')); break;
                    case 'l': getStack1(); stack.push(GlobalVariables.value('l')); break;
                    case 'm': getStack1(); stack.push(GlobalVariables.value('m')); break;
                    case 'n': getStack1(); stack.push(GlobalVariables.value('n')); break;
                    case 'o': getStack1(); stack.push(GlobalVariables.value('o')); break;
                    case 'p': getStack1(); stack.push(GlobalVariables.value('p')); break;
                    case 'q': getStack1(); stack.push(GlobalVariables.value('q')); break;
                    case 'r': getStack1(); stack.push(GlobalVariables.value('r')); break;
                    case 's': getStack1(); stack.push(GlobalVariables.value('s')); break;
                    case 'u': getStack1(); stack.push(GlobalVariables.value('u')); break;
                    case 'v': getStack1(); stack.push(GlobalVariables.value('v')); break;
                    case 'x': getStack1(); stack.push(GlobalVariables.value('x')); break;
                    case 'y': getStack1(); stack.push(GlobalVariables.value('y')); break;
                    case 'z': getStack1(); stack.push(GlobalVariables.value('z')); break;

					//定数
                    case 'pi': getStack1(); stack.push(Math.PI); break;
                    case 'e': getStack1(); stack.push(Math.E); break;
                    case 'w': getStack1(); stack.push(Main.stimuliStage.rectangle.width); break;
                    case 'h': getStack1(); stack.push(Main.stimuliStage.rectangle.height); break;

                    default: stack.push(i); break;
                }
            }

			value = Number(stack.pop());
			
            return value;
        }

        private function checkMinus():void
        {
            var tmp:String;
            // 先頭のマイナスを数値に反映させる
            if (items[0] == "-") {
                items[1] = items[1] * -1;
                items.shift();
            }
            // 演算子の直後のマイナスを数値に反映させる
            var enzansi:String = "()+-*/%^";
            var i:int = 0;
            while ( i < items.length ) {
                if (enzansi.indexOf(items[i]) >= 0) {
                    if (items[i+1] == "-") {
                        items[i+2] = items[i+2] * -1;
                        items.splice(i+1, 1);
                    }
                }
                i++;
            }
        }
        /**
        * 文字列の計算式を計算して返す
        * @param    e    文字列で計算式を指定する(中置記法)
        * @rerurn        計算結果
        */
        public function CalcInfix(e:String, vName:String)
        {
            // 数値と演算子を区切る
			targetString = new String(e);

			e = toUpper(e);
			e = replaceVariables(e);
			
			if (e.indexOf(vName) != -1) {
				value = NaN;
				return;
			}
            
			e = e.replace(/\s*/g, ''); // 空白を除去
            e = e.replace(/((\d+\.\d+|\d+))/g, '$1 '); // 数値の後にスペース
            e = e.replace(/([a-z\_]+)/gi, '$1 '); // 関数の後にスペース
            e = e.replace(/([\%\(\)\-\^\+\*\/\^])/g, '$1 ');
            
			items = e.split(/\s/);
            checkMinus();
			expression();
			eval();
        }
		
		private function toUpper(e:String):String {

            e = e.replace(/irand/g, "IRAND");
			
            e = e.replace(/floor/g, "FLOOR");
            e = e.replace(/ceil/g, "CEIL");
            e = e.replace(/rand/g, "RAND");
            e = e.replace(/sin/g, "SIN");
            e = e.replace(/cos/g, "COS");
            e = e.replace(/tan/g, "TAN");
            e = e.replace(/asin/g, "ASIN");
            e = e.replace(/acos/g, "ACOS");
            e = e.replace(/atan/g, "ATAN");
            e = e.replace(/abs/g, "ABS");
            e = e.replace(/exp/g, "EXP");
            e = e.replace(/ln/g, "LN");
            e = e.replace(/sqrt/g, "SQRT");
            e = e.replace(/round/g, "ROUND");
//            e = e.replace(/rect/g, "RECT");
			return e;
		}
		
		private function replaceVariables(e:String):String {
			
			for (var i:uint = 0; i < VARIABLE.length; i++) {
				var reg:RegExp = new RegExp(VARIABLE.charAt(i), "g");
				e = e.replace(reg, VARIABLE.charAt(i) + "(0)");
			}
//            e = e.replace(/t/gi, "t(0)");
            e = e.replace(/pi/g, "pi(0)");
//            e = e.replace(/e/gi, "e(0)");
//            e = e.replace(/w/gi, "w(0)");
//            e = e.replace(/h/gi, "h(0)");

			return e;
			
		}
    }
}
