﻿package  
{
	import fl.controls.ColorPicker;
	import fl.controls.Label;
	import fl.controls.TextInput;
	import fl.events.ColorPickerEvent;
	import flash.events.Event;
	import flash.events.TextEvent;
	import flash.geom.Point;
	
	/**
	 * ...
	 * @author ...
	 */
	public class Gabor extends Stimulus
	{
		static private var defaultSize:Point = new Point(160, 160);
		
		private var sizeX:ParameterControl;
		private var sizeY:ParameterControl;
		
		private var redA:ParameterControl;
		private var greenA:ParameterControl;
		private var blueA:ParameterControl;

		private var redB:ParameterControl;
		private var greenB:ParameterControl;
		private var blueB:ParameterControl;
		
		
		private var amp:ParameterControl;
		private var fs:ParameterControl;
		private var ft:ParameterControl;
		
		private var gaussAmp:ParameterControl;
		

		private function paramCopy():void {
			
			sizeX = paramList[0];
			sizeY = paramList[1];
			redA = paramList[2];
			greenA = paramList[3];
			blueA = paramList[4];
			redB = paramList[5];
			greenB = paramList[6];
			blueB = paramList[7];
			amp = paramList[8];
			fs = paramList[9];
			ft = paramList[10];
			gaussAmp = paramList[11];
			positionX = paramList[12];
			positionY = paramList[13];
			alphaControl = paramList[14];
			rotControl = paramList[15];
			scaleXControl = paramList[16];
			scaleYControl = paramList[17];
		}

		
		public function Gabor(name:String, cpy:Boolean = false) 
		{
			paramList = new Array();
			paramComboBox = new MyComboBox(10, 4, onParamFocus, name + "'s parameter");

			if(!cpy) {
				paramList.push(sizeX = new ParameterControl(defaultRow, "width(t)", defaultSize.x, 0.0, 1000, this));
				paramList.push(sizeY = new ParameterControl(defaultRow, "height(t)", defaultSize.y, 0.0, 1000, this));

				paramList.push(redA = new ParameterControl(defaultRow, "red_1(t)", 255, 0.0, 255, this));
				paramList.push(greenA = new ParameterControl(defaultRow, "green_1(t)", 255, 0.0, 255, this));
				paramList.push(blueA = new ParameterControl(defaultRow, "blue_1(t)", 255, 0.0, 255, this));

				paramList.push(redB = new ParameterControl(defaultRow, "red_2(t)", 0, 0.0, 255, this));
				paramList.push(greenB = new ParameterControl(defaultRow, "green_2(t)", 0, 0.0, 255, this));
				paramList.push(blueB = new ParameterControl(defaultRow, "blue_2(t)", 0, 0.0, 255, this));

				paramList.push(amp = new ParameterControl(defaultRow, "grating amp(t)", 1, 0, 1, this));
				paramList.push(fs = new ParameterControl(defaultRow, "fs(t)", 0.4, 0, 100, this));
				paramList.push(ft = new ParameterControl(defaultRow, "ft(t)", 10, -1000, 1000, this));

				paramList.push(gaussAmp = new ParameterControl(defaultRow, "gauss amp(t)", 1, 0, 1, this));

			}
			
			super(name, cpy);			

			if(!cpy)
				repaint();

		}
				
		override internal function clone(i:String):void {
			
			var cpy:Gabor = new Gabor(instanceName + i, true);
			
			for each(var v:ParameterControl in paramList) {
				cpy.paramList.push(v.clone(cpy));
			}

			cpy.paramCopy();
			Main.controlPanel.instanceBox.add(cpy.instanceName);
			
			cpy.repaint();
		}
		
		private function getColor(r:uint, g:uint, b:uint):uint {
			
			return ((r << 16) | (g << 8) | b);
		}
		
		override internal function repaint():void {
			
			step();
			
			drawGabor();
		}
		
		private function drawGabor():void {
	
			var widthG:uint = sizeX.value();
			var heightG:uint = sizeY.value();

			var amplitude:Number = amp.value();
			var spatioF:Number = fs.value();
			var temporalF:Number = ft.value();

			var rA:Number = redA.value() / 2;
			var gA:Number = greenA.value() / 2;
			var bA:Number = blueA.value() / 2;
			var rB:Number = redB.value() / 2;
			var gB:Number = greenB.value() / 2;
			var bB:Number = blueB.value() / 2;
			
			var rRange:Number = (rB - rA) * amplitude;
			var gRange:Number = (gB - gA) * amplitude;
			var bRange:Number = (bB - bA) * amplitude;

			var gaussAmplitude:Number = gaussAmp.value();
			

			var tmp:Number = Stimulus.t * temporalF;
			
			var sx2:Number = Math.pow(widthG / 6, 2);
			var sy2:Number = Math.pow(heightG / 6, 2);

			for (var i:int = -widthG / 2; i < widthG / 2; i++ )
				for (var j:int = -heightG / 2; j < heightG / 2; j++ ) {

					var s:Number = spatioF * i;
						
					var color:uint = getColor(rA + rRange * Math.sin(s + tmp), gA + gRange * Math.sin(s + tmp), bA + bRange * Math.sin(s + tmp));
					var alfa:Number = Math.exp( -(i * i / sx2 + j * j / sy2)) * gaussAmplitude;
					graphics.beginFill(color, alfa);
					graphics.drawRect(i, j, 1, 1);
					graphics.endFill();
				}
			
		}
	}	
}
