﻿package  
{
	import fl.controls.ColorPicker;
	import fl.controls.ComboBox;
	import fl.controls.Label;
	import fl.controls.TextInput;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	
	/**
	 * ...
	 * @author ...
	 */
	public class Stimulus extends Sprite
	{
		static public const CIRCLE:uint = 0;
		static public const RECTANGLE:uint = 1;
		static public const GRATING:uint = 2;
		static public const CIRCLAR_GRATING:uint = 3;
		static public const IMAGE:uint = 4;
		static public const GABOR:uint = 5;
		static public const GAUSS:uint = 6;
		static public const GAUSS_FILTER:uint = 7;

		static public const STIMULI:Array = ["circle", "rectangle", "grating",
										"circular grating", "image", "gabor", "gauss", "gauss filter"];

		static internal var defaultRow:uint = 6;
		static public const defaultColor:uint = 0x000000;
		static internal var t:Number = 0;

		internal var instanceName:String;
		
		internal var paramList:Array;		
		internal var paramComboBox:MyComboBox;

		internal var currentControl:ParameterControl;
		
		internal var positionX:ParameterControl;
		internal var positionY:ParameterControl;
				
		internal var alphaControl:ParameterControl;
		
		internal var rotControl:ParameterControl;
		
		internal var scaleXControl:ParameterControl;
		internal var scaleYControl:ParameterControl;
		
				
		public function Stimulus(name:String, copy:Boolean = false) 
		{	
			this.name = instanceName = name;
			
			if(!copy) {
				paramInit();
			}
			Main.stimuliStage.addChild(this);
			paramComboBox.addEventListener(Event.CHANGE, onClose);
			paramComboBox.rowCount = 100;
			
			addEventListener(MouseEvent.MOUSE_OVER, onMouseOver);
			addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			
			buttonMode = true;

		}
		
		internal function clone(i:String):void {
			//オーバーライド
		}
	
		internal function onClose(event:Event):void {
			paramComboBox.reset();
		}
		
		private function initComboBox():void {

			for each(var v:ParameterControl in paramList) {
				paramComboBox.add(v.label.text.substring(0, v.label.text.length - 3));
			}
		}
		
		
		private function paramInit():void {

			paramList.push(positionX = new ParameterControl(defaultRow, "x(t)", Main.stimuliStage.rectangle.width / 2, 0.0, Main.stimuliStage.stage.stageWidth, this));
			paramList.push(positionY = new ParameterControl(defaultRow, "y(t)", Main.stimuliStage.rectangle.height / 2, 0.0, Main.stimuliStage.stage.stageHeight, this));
			
			paramList.push(alphaControl = new ParameterControl(defaultRow, "alpha(t)", 1.0, 0.0, 1.0, this));
			
			paramList.push(rotControl = new ParameterControl(defaultRow, "angle(t)", 0.0, -180, 360, this));
			
			paramList.push(scaleXControl = new ParameterControl(defaultRow, "scaleX(t)", 1.0, 0, 100, this));
			paramList.push(scaleYControl = new ParameterControl(defaultRow, "scaleY(t)", 1.0, 0, 100, this));
		}
		
				
		internal function step():void {

			graphics.clear();
			x = positionX.value();
			y = positionY.value();
			alpha = alphaControl.value();
			rotation = rotControl.value();			
			scaleX = scaleXControl.value();
			scaleY = scaleYControl.value();
			
		}
		
		internal function removeControl():void {

			if(currentControl != null)			
				currentControl.remove();
			
			Main.controlPanel.removeChild(paramComboBox);			
		}
		
		internal function syntaxErrorCheck():Boolean {
			
			for each(var v:ParameterControl in paramList)
				if (v.syntaxErrorCheck())
					return true;
			
			return false;
					
		}
		
		internal function addControl():void {

			if(currentControl != null)
				currentControl.add();
			
			Main.controlPanel.addChild(paramComboBox);			
		}
		
		internal function onParamFocus(event:Event):void {

			if (currentControl != null)
				currentControl.remove();
				
			currentControl = paramList[paramComboBox.selectedIndex];

			currentControl.add();

		}
		
		internal function repaint():void {
			//over ride用
		}
		
		
		private function onMouseDown(event:MouseEvent):void {
			startDrag(false, Main.stimuliStage.rectangle);
			addEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
			Main.stimuliStage.stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}		
		
		private function onMouseMove(event:MouseEvent):void {
			event.updateAfterEvent();
			positionX.textInput.text = x.toString();
			positionY.textInput.text = y.toString();
		}
		
		private function onMouseUp(event:MouseEvent):void {
			stopDrag();
			removeEventListener(MouseEvent.MOUSE_MOVE, onMouseMove);
			Main.stimuliStage.stage.removeEventListener(MouseEvent.MOUSE_UP, onMouseUp);			
			
			positionX.eval(null);
			positionY.eval(null);
		}
		
		private function onMouseOver(event:MouseEvent):void {
			addEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
			addEventListener(MouseEvent.MOUSE_OUT, onMouseOut);			
		}
		
		private function onMouseOut(event:MouseEvent):void {
			removeEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
			removeEventListener(MouseEvent.MOUSE_OUT, onMouseOut);			
			
			scaleXControl.eval(null);
			scaleYControl.eval(null);
			rotControl.eval(null);
		}
		
		private function onMouseWheel(event:MouseEvent):void {
			
			if (event.ctrlKey) {
				scaleX += event.delta / 50;
				scaleXControl.textInput.text = scaleX.toString();
			}
			else if (event.shiftKey) {
				scaleY += event.delta / 50;
				scaleYControl.textInput.text = scaleY.toString();
			}			
//			else if (event.shiftKey) {
//				rotation += event.delta / 10;
//			}
			else {
				rotation += event.delta / 2;
				rotControl.textInput.text = rotation.toString();
			}
		}
		
	}
	
}

